@@ -1,7 +1,6 @@
 import asyncio
 import csv
 import inspect
-import json
 import logging
 import re
 import sys
@@ -23,6 +22,7 @@ from typing import (
     Union,
     cast,
 )
+from urllib import parse
 from uuid import uuid4
 
 import click
@@ -30,10 +30,9 @@ import coloredlogs  # type: ignore
 import discord
 import discord.errors
 import hupper  # type: ignore
+import redis
 import requests
 from aiohttp import web
-from aiohttp.web_app import Application
-from discord.channel import TextChannel
 from dotenv import load_dotenv
 from sqlalchemy import exc
 from sqlalchemy.orm.session import Session
@@ -63,7 +62,10 @@ from spellbot.data import (
     UserTeam,
 )
 from spellbot.operations import (
+    ChannelType,
     safe_clear_reactions,
+    safe_create_category_channel,
+    safe_create_invite,
     safe_create_voice_channel,
     safe_delete_channel,
     safe_delete_message,
@@ -195,7 +197,7 @@ def post_link(server_xid: int, channel_xid: int, message_xid: int) -> str:
 
 
 def is_admin(
-    channel: TextChannel, user_or_member: Union[discord.User, discord.Member]
+    channel: discord.TextChannel, user_or_member: Union[discord.User, discord.Member]
 ) -> bool:
     """Checks to see if given user or member has the admin role on this server."""
     member = (
@@ -277,12 +279,11 @@ class SpellBot(discord.Client):
         self,
         token: Optional[str] = None,
         auth: Optional[str] = None,
-        metrics_auth: str = "",
         db_url: str = DEFAULT_DB_URL,
+        redis_url: Optional[str] = None,
         log_level: Union[int, str] = logging.ERROR,
         mock_games: bool = False,
         loop: AbstractEventLoop = None,
-        app: Application = None,
     ):
         coloredlogs.install(
             level=log_level,
@@ -311,10 +312,20 @@ class SpellBot(discord.Client):
         self.auth = auth or ""
         self.mock_games = mock_games
 
-        # Metrics endpoint is only added if auth key and web app are given.
-        self.metrics_auth = metrics_auth
-        if app and self.metrics_auth:
-            app.router.add_get("/metrics", self._metrics)
+        # Connect to Redis Cloud if we have a URL for it.
+        self.metrics_db: Optional[redis.Redis] = None
+        if redis_url:
+            url = parse.urlparse(redis_url)
+            if url.hostname and url.port:
+                try:
+                    self.metrics_db = redis.Redis(  # type: ignore
+                        host=url.hostname,
+                        port=url.port,
+                        password=url.password,
+                        health_check_interval=30,  # type: ignore
+                    )
+                except redis.exceptions.RedisError as e:
+                    logger.exception("redis error: %s", e)
 
         # We have to make sure that DB_DIR exists before we try to create
         # the database as part of instantiating the Data object.
@@ -335,20 +346,6 @@ class SpellBot(discord.Client):
             member[0][9:] for member in members if member[0].startswith("spellbot_")
         ]
 
-        self._begin_background_tasks(loop)
-
-    async def _metrics(self, request):  # pragma: no cover
-        if request.headers["Authorization"] != self.metrics_auth:
-            return web.Response(status=403)
-
-        async with self.session() as session:
-            data = {
-                "servers": session.query(Server).count(),
-                "games": session.query(Game).filter(Game.status == "started").count(),
-                "users": session.query(User).count(),
-            }
-            return web.Response(text=json.dumps(data))
-
     @asynccontextmanager
     async def session(self) -> AsyncGenerator[Session, None]:
         session = self.data.Session()
@@ -367,6 +364,7 @@ class SpellBot(discord.Client):
         """Start up any periodic background tasks."""
         self.cleanup_expired_games_task(loop)
         self.cleanup_old_voice_channels_task(loop)
+        self.update_metrics_task(loop)
 
         # TODO: Make this manually triggered.
         # self.cleanup_started_games_task(loop)
@@ -375,11 +373,11 @@ class SpellBot(discord.Client):
         self, loop: asyncio.AbstractEventLoop
     ) -> None:  # pragma: no cover
         """Starts a task that culls old games."""
-        THIRTY_SECONDS = 30
+        TWO_MINUTES = 120
 
         async def task() -> None:
             while True:
-                await asyncio.sleep(THIRTY_SECONDS)
+                await asyncio.sleep(TWO_MINUTES)
                 await self.cleanup_expired_games()
 
         loop.create_task(task())
@@ -412,12 +410,12 @@ class SpellBot(discord.Client):
         self, loop: asyncio.AbstractEventLoop
     ) -> None:  # pragma: no cover
         """Starts a task that deletes old voice channels."""
-        THIRTY_MINUTES = 1800
+        TEN_MINUTES = 600
 
         async def task() -> None:
             while True:
-                await asyncio.sleep(THIRTY_MINUTES)
                 await self.cleanup_old_voice_channels()
+                await asyncio.sleep(TEN_MINUTES)
 
         loop.create_task(task())
 
@@ -432,10 +430,13 @@ class SpellBot(discord.Client):
                 )
                 if not chan:
                     game.voice_channel_xid = None  # type: ignore
+                    game.voice_channel_invite = None  # type: ignore
                     continue
-                if not chan.members:  # type: ignore
+
+                if not chan.voice_states.keys():  # type: ignore
                     await safe_delete_channel(chan, game.guild_xid)
                     game.voice_channel_xid = None  # type: ignore
+                    game.voice_channel_invite = None  # type: ignore
             session.commit()
 
     def cleanup_started_games_task(
@@ -461,6 +462,42 @@ class SpellBot(discord.Client):
                 session.delete(game)
             session.commit()
 
+    def update_metrics_task(
+        self, loop: asyncio.AbstractEventLoop
+    ) -> None:  # pragma: no cover
+        """Starts a task that updates some metrics about this bot."""
+        if not self.metrics_db:
+            return
+
+        ONE_HOUR = 3600
+
+        async def task() -> None:
+            while True:
+                await self.update_metrics()
+                await asyncio.sleep(ONE_HOUR)
+
+        loop.create_task(task())
+
+    async def update_metrics(self) -> None:
+        if not self.metrics_db:
+            return
+
+        logger.info("starting update metrics task...")
+        async with self.session() as session:
+            games = session.query(Game).filter(Game.status == "started").count()
+            servers = session.query(Server).count()
+            users = session.query(User).count()
+            try:
+                self.metrics_db.mset(
+                    {
+                        "servers": servers,
+                        "games": games,
+                        "users": users,
+                    }
+                )
+            except redis.exceptions.RedisError as e:
+                logger.exception("redis error: %s", e)
+
     @property
     def commands(self) -> List[str]:
         """Returns the list of commands supported by this bot."""
@@ -482,11 +519,28 @@ class SpellBot(discord.Client):
         r = requests.post(CREATE_ENDPOINT, headers=headers)
         return cast(str, r.json()["gameUrl"])
 
-    async def make_voice(self, game: Game) -> Optional[int]:
-        return await safe_create_voice_channel(
-            self, game.server.guild_xid, f"Game-SB{game.id}"
+    async def setup_voice(self, session: Session, game: Game) -> None:
+        """Adds voice information to the game object. DOES NOT COMMIT!"""
+        if not game.server.create_voice:
+            return
+
+        category = await self.ensure_server_voice_category(session, game.server)
+
+        # if category is None, then we'll simply create a non-categorized channel
+        voice_channel = await safe_create_voice_channel(
+            self, game.server.guild_xid, f"Game-SB{game.id}", category=category
         )
 
+        if voice_channel:
+            game.voice_channel_xid = voice_channel.id  # type: ignore
+
+            TEN_MINUTES = 600  # Make this configurable?
+            voice_invite = await safe_create_invite(
+                voice_channel, game.guild_xid, max_age=TEN_MINUTES
+            )
+            if voice_invite:
+                game.voice_channel_invite = voice_invite  # type: ignore
+
     def ensure_user_exists(
         self, session: Session, user: Union[discord.User, discord.Member]
     ) -> User:
@@ -511,6 +565,31 @@ class SpellBot(discord.Client):
             session.commit()
         return cast(Server, server)
 
+    async def ensure_server_voice_category(
+        self, session: Session, server: Server
+    ) -> Optional[discord.CategoryChannel]:
+        category: Optional[ChannelType] = None
+
+        if not server.create_voice:
+            return None
+
+        if server.voice_category_xid:
+            category = await safe_fetch_channel(
+                self, server.voice_category_xid, server.guild_xid
+            )
+        if category:
+            return cast(discord.CategoryChannel, category)
+
+        category = await safe_create_category_channel(
+            self, server.guild_xid, "SpellBot Voice Channels"
+        )
+        if category:
+            server.voice_category_xid = category.id  # type: ignore
+            session.commit()
+            return cast(discord.CategoryChannel, category)
+
+        return None
+
     async def try_to_delete_message(self, game: Game) -> None:
         """Attempts to remove a ✋ from the given game message for the given user."""
         if not game.channel_xid:
@@ -686,8 +765,7 @@ class SpellBot(discord.Client):
                 game.url = (
                     self.create_spelltable_url() if game.system == "spelltable" else None
                 )
-                if game.server.create_voice:
-                    game.voice_channel_xid = await self.make_voice(game)
+                await self.setup_voice(session, game)
                 game.status = "started"
                 game.game_power = game.power
                 session.commit()
@@ -742,6 +820,7 @@ class SpellBot(discord.Client):
     async def on_ready(self) -> None:
         """Behavior when the client has successfully connected to Discord."""
         logger.debug("logged in as %s", self.user)
+        self._begin_background_tasks(self.loop)
 
     ##############################
     # Bot Command Functions
@@ -942,8 +1021,7 @@ class SpellBot(discord.Client):
                     self.create_spelltable_url() if game.system == "spelltable" else None
                 )
                 game.url = game_url  # type: ignore
-                if game.server.create_voice:
-                    game.voice_channel_xid = await self.make_voice(game)  # type: ignore
+                await self.setup_voice(session, game)
                 game.status = "started"  # type: ignore
                 game.game_power = game.power  # type: ignore
                 session.commit()
@@ -972,6 +1050,9 @@ class SpellBot(discord.Client):
         tag_names: List[str] = opts["tags"]
         system: str = opts["system"]
 
+        if not server.tags_enabled:
+            tag_names = []
+
         if not await self._validate_size(message, size):
             await safe_react_error(message)
             return
@@ -1532,8 +1613,7 @@ class SpellBot(discord.Client):
                 self.create_spelltable_url() if game.system == "spelltable" else None
             )
             game.url = game_url  # type: ignore
-            if game.server.create_voice:
-                game.voice_channel_xid = await self.make_voice(game)  # type: ignore
+            await self.setup_voice(session, game)
             game.status = "started"  # type: ignore
             game.game_power = game.power  # type: ignore
             response = game.to_embed(dm=True)
@@ -1638,9 +1718,7 @@ class SpellBot(discord.Client):
         now = datetime.utcnow()
         expires_at = now + timedelta(minutes=server.expire)
         url = self.create_spelltable_url() if system == "spelltable" else None
-        # FIXME: Do we create the voice channel in this case?
-        # if game.server.create_voice:
-        #     game.voice_channel_xid = await self.make_voice(game)  # type: ignore
+        # NOTE: Don't automatically create a voice channel in this case...
         game = Game(
             channel_xid=message.channel.id,
             created_at=now,
@@ -1714,7 +1792,7 @@ class SpellBot(discord.Client):
                         self, int(channel_xid), message.channel.guild.id
                     )
                     if channel:
-                        name = cast(TextChannel, channel).name
+                        name = cast(discord.TextChannel, channel).name
                         channel_name_cache[channel_xid] = f"#{name}"
                     else:
                         channel_name_cache[channel_xid] = f"<#{channel_xid}>"
@@ -1901,9 +1979,10 @@ class SpellBot(discord.Client):
         * `prefix <string>`: Set SpellBot's command prefix for text channels.
         * `links <private | public>`: Set the privacy for generated SpellTable links.
         * `expire <number>`: Set the number of minutes before pending games expire.
-        * `teams <list>`: Sets the teams available on this server.
+        * `teams <list | none>`: Sets the teams available on this server.
         * `power <on | off>`: Turns the power command on or off for this server.
         * `voice <on | off>`: When on, SpellBot will automatically create voice channels.
+        * `tags <on | off>`: Turn on or off the ability to use tags on your server.
         * `help`: Get detailed usage help for SpellBot.
         & <subcommand> [subcommand parameters]
         """
@@ -1943,6 +2022,8 @@ class SpellBot(discord.Client):
             await self.spellbot_power(session, server, params[1:], message)
         elif command == "voice":
             await self.spellbot_voice(session, server, params[1:], message)
+        elif command == "tags":
+            await self.spellbot_tags(session, server, params[1:], message)
         else:
             await message.channel.send(
                 s(
@@ -2252,6 +2333,40 @@ class SpellBot(discord.Client):
         )
         await safe_react_ok(message)
 
+    async def spellbot_tags(
+        self,
+        session: Session,
+        server: Server,
+        params: List[str],
+        message: discord.Message,
+    ) -> None:
+        if not params or params[0].lower() not in ["on", "off"]:
+            await message.channel.send(
+                s(
+                    "spellbot_tags_bad",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                )
+            )
+            await safe_react_error(message)
+            return
+
+        server = (
+            session.query(Server)
+            .filter(Server.guild_xid == message.channel.guild.id)
+            .one_or_none()
+        )
+        setting = params[0].lower()
+        server.tags_enabled = setting == "on"  # type: ignore
+        session.commit()
+        await message.channel.send(
+            s(
+                "spellbot_tags",
+                reply=f"<@{cast(discord.User, message.author).id}>",
+                setting=setting,
+            )
+        )
+        await safe_react_ok(message)
+
     async def spellbot_config(
         self,
         session: Session,
@@ -2272,6 +2387,7 @@ class SpellBot(discord.Client):
         embed.add_field(name="Active channels", value=channels_str)
         embed.add_field(name="Links", value=server.links.title())
         embed.add_field(name="Power", value="On" if server.power_enabled else "Off")
+        embed.add_field(name="Tags", value="On" if server.tags_enabled else "Off")
         embed.add_field(
             name="Voice Channels",
             value="On" if server.create_voice else "Off",
@@ -2320,18 +2436,17 @@ def get_host(fallback: str) -> str:  # pragma: no cover
     return value or fallback
 
 
-def get_metrics_auth(fallback: str) -> str:  # pragma: no cover
-    """Returns the metrics auth key from the environment or else the given fallback."""
-    value = getenv("SPELLBOT_METRICS_AUTH", fallback)
-    return value or fallback
-
-
 def get_log_level(fallback: str) -> str:  # pragma: no cover
     """Returns the log level from the environment or else the given gallback."""
     value = getenv("SPELLBOT_LOG_LEVEL", fallback)
     return value or fallback
 
 
+def get_redis_url() -> Optional[str]:  # pragma: no cover
+    """Gets redis cloud url if your REDISCLOUD_URL env var has been set."""
+    return getenv("REDISCLOUD_URL", None)
+
+
 async def ping(request):  # pragma: no cover
     return web.Response(text="ok")
 
@@ -2391,14 +2506,6 @@ async def ping(request):  # pragma: no cover
         "you can also set this via the SPELLBOT_HOST environment variable."
     ),
 )
-@click.option(
-    "--metrics-auth",
-    default="",
-    help=(
-        "Metrics endpoint HTTP Authorization header key to use; "
-        "you can also set this via the SPELLBOT_METRICS_AUTH environment variable."
-    ),
-)
 @click.version_option(version=__version__)
 @click.option(
     "--dev",
@@ -2420,7 +2527,6 @@ def main(
     port: int,
     port_env: str,
     host: str,
-    metrics_auth: str,
     dev: bool,
     mock_games: bool,
 ) -> None:  # pragma: no cover
@@ -2433,8 +2539,8 @@ def main(
     port_env = get_port_env(port_env)
     port = get_port(port_env, port)
     host = get_host(host)
-    metrics_auth = get_metrics_auth(metrics_auth)
     log_level = get_log_level(log_level)
+    redis_url = get_redis_url()
 
     # We have to make sure that application directories exist
     # before we try to create we can run any of the migrations.
@@ -2462,12 +2568,11 @@ def main(
     client = SpellBot(
         token=token,
         auth=auth,
-        metrics_auth=metrics_auth,
         db_url=database_url,
+        redis_url=redis_url,
         log_level="DEBUG" if verbose else log_level,
         mock_games=mock_games,
         loop=loop,
-        app=app,
     )
 
     runner = web.AppRunner(app)
